<?php

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );
require_once  'constants.php';
require_once  'nitropack-sdk/autoload.php';
$np_originalRequestCookies = $_COOKIE;

function nitropack_passes_cookie_requirements() {
    $cookieStr = implode("|", array_keys($_COOKIE));
    $safeCookie = strpos($cookieStr, "comment_author") === false && strpos($cookieStr, "wp-postpass_") === false && empty($_COOKIE["woocommerce_items_in_cart"]);
    $loginCookies = array(defined('NITROPACK_LOGGED_IN_COOKIE') ? NITROPACK_LOGGED_IN_COOKIE : (defined('LOGGED_IN_COOKIE') ? LOGGED_IN_COOKIE : ''));
    $isUserLoggedIn = false;

    foreach ($loginCookies as $loginCookie) {
        if (!empty($_COOKIE[$loginCookie])) {
            $isUserLoggedIn = true;
            break;
        }
    }

    return $safeCookie && !$isUserLoggedIn;
}

function nitropack_activate() {
    nitropack_set_wp_cache_const(true);
    $htaccessFile = nitropack_trailingslashit(NITROPACK_DATA_DIR) . ".htaccess";
    if (!file_exists($htaccessFile) && nitropack_init_data_dir()) {
        file_put_contents($htaccessFile, "deny from all");
    }
    nitropack_install_advanced_cache();

    try {
        do_action('nitropack_integration_purge_all');
    } catch (\Exception $e) {
        // Exception while signaling our 3rd party integration addons to purge their cache
    }

    if (nitropack_is_connected()) {
        nitropack_event("enable_extension");
    } else {
        setcookie("nitropack_after_activate_notice", 1, time() + 3600);
    }
}

function nitropack_deactivate() {
    nitropack_set_wp_cache_const(false);
    nitropack_uninstall_advanced_cache();

    try {
        do_action('nitropack_integration_purge_all');
    } catch (\Exception $e) {
        // Exception while signaling our 3rd party integration addons to purge their cache
    }

    if (nitropack_is_connected()) {
        nitropack_event("disable_extension");
    }
}

function nitropack_install_advanced_cache() {
    $templatePath = nitropack_trailingslashit(__DIR__) . "advanced-cache.php";
    if (file_exists($templatePath)) {
        $contents = file_get_contents($templatePath);
        $contents = str_replace("/*NITROPACK_FUNCTIONS_FILE*/", __FILE__, $contents);
        $contents = str_replace("/*LOGIN_COOKIES*/", '"' . LOGGED_IN_COOKIE . '"', $contents);

        $advancedCacheFile = nitropack_trailingslashit(WP_CONTENT_DIR) . 'advanced-cache.php';
        if (WP_DEBUG) {
            return file_put_contents($advancedCacheFile, $contents);
        } else {
            return @file_put_contents($advancedCacheFile, $contents);
        }
    }
}

function nitropack_uninstall_advanced_cache() {
    $advancedCacheFile = nitropack_trailingslashit(WP_CONTENT_DIR) . 'advanced-cache.php';
    if (file_exists($advancedCacheFile)) {
        if (WP_DEBUG) {
            return file_put_contents($advancedCacheFile, "");
        } else {
            return @file_put_contents($advancedCacheFile, "");
        }
    }
}

function nitropack_set_wp_cache_const($status) {
    if (nitropack_is_flywheel()) { // Flywheel: This is configured throught the FW control panel
        return true;
    }

    $configFilePath = nitropack_trailingslashit(ABSPATH) . "wp-config.php";
    if (!file_exists($configFilePath)) {
        $configFilePath = nitropack_trailingslashit(dirname(ABSPATH)) . "wp-config.php";
        $settingsFilePath = nitropack_trailingslashit(dirname(ABSPATH)) . "wp-settings.php"; // We need to check for this file to avoid confusion if the current installation is a nested directory of another WP installation. Refer to wp-load.php for more information.
        if (!file_exists($configFilePath) || !is_writable($configFilePath) || file_exists($settingsFilePath)) {
            return false;
        }
    }

    $newVal = sprintf("define( 'WP_CACHE', %s); // Modified by NitroPack\n", ($status ? "true" : "false") );
    $lines = file($configFilePath);
    $wpCacheFound = false;

    foreach ($lines as &$line) {
        if (preg_match("/define\s*\(\s*['\"](.*?)['\"]/", $line, $matches)) {
            if ($matches[1] == "WP_CACHE") {
                $line = $newVal;
                $wpCacheFound = true;
                break;
            }
        }
    }

    if (!$wpCacheFound) {
        if (!$status) return true; // No need to modify the file at all
        array_splice($lines, 1, 0, [$newVal]);
    }

    return WP_DEBUG ? file_put_contents($configFilePath, implode("", $lines)) : @file_put_contents($configFilePath, implode("", $lines));
}

function is_valid_nitropack_webhook() {
    return !empty($_GET["nitroWebhook"]) && !empty($_GET["token"]) && nitropack_validate_webhook_token($_GET["token"]);
}

function is_valid_nitropack_beacon() {
    if (!isset($_POST["nitroBeaconUrl"]) || !isset($_POST["nitroBeaconHash"])) return false;

    $siteConfig = nitropack_get_site_config();
    if (!$siteConfig || empty($siteConfig["siteSecret"])) return false;
    
    if (function_exists("hash_hmac") && function_exists("hash_equals")) {
        $url = base64_decode($_POST["nitroBeaconUrl"]);
        $localHash = hash_hmac("sha512", $url, $siteConfig["siteSecret"]);
        return hash_equals($_POST["nitroBeaconHash"], $localHash);
    } else {
        return !empty($_POST["nitroBeaconUrl"]);
    }
}

function nitropack_handle_beacon() {
    $siteConfig = nitropack_get_site_config();
    if ($siteConfig && !empty($siteConfig["siteId"]) && !empty($siteConfig["siteSecret"]) && !empty($_POST["nitroBeaconUrl"])) {
        $url = base64_decode($_POST["nitroBeaconUrl"]);

        if (null !== $nitro = get_nitropack_sdk($siteConfig["siteId"], $siteConfig["siteSecret"], $url) ) {
            try {
                if (!$nitro->hasLocalCache(false)) {
                    header("X-Nitro-Beacon: FORWARD");
                    $hasCache = $nitro->hasRemoteCache("default", false); // Download the new cache file
                    $nitro->purgeProxyCache($url);
                    do_action('nitropack_integration_purge_url', $url);
                } else {
                    header("X-Nitro-Beacon: SKIP");
                }
            } catch (Exception $e) {
                // not a critical error, do nothing
            }
        }
    }
    exit;
}

function nitropack_handle_webhook() {
    $siteConfig = nitropack_get_site_config();
    if ($siteConfig && $siteConfig["webhookToken"] == $_GET["token"]) {
        switch($_GET["nitroWebhook"]) {
        case "config":
            nitropack_fetch_config();
            break;
        case "cache_ready":
            if (!empty($_POST["url"])) {
                $readyUrl = NULL;
                if (!function_exists("esc_url")) {
                    $sanitizedUrl = filter_var($_POST["url"], FILTER_SANITIZE_URL);
                    if ($sanitizedUrl !== false && filter_var($sanitizedUrl, FILTER_VALIDATE_URL) !== false) {
                        $readyUrl = $sanitizedUrl;
                    }
                } else if ($validatedUrl = esc_url($_POST["url"], array("http", "https"), "notdisplay")) {
                    $readyUrl = $validatedUrl;
                }

                if ($readyUrl && null !== $nitro = get_nitropack_sdk($siteConfig["siteId"], $siteConfig["siteSecret"], $readyUrl) ) {
                    $hasCache = $nitro->hasRemoteCache("default", false); // Download the new cache file
                    $nitro->purgeProxyCache($readyUrl);
                    do_action('nitropack_integration_purge_url', $readyUrl);
                }
            }
            break;
        case "cache_clear":
            if (!empty($_POST["url"])) {
                if (!function_exists("esc_url")) {
                    $sanitizedUrl = filter_var($_POST["url"], FILTER_SANITIZE_URL);
                    if ($sanitizedUrl !== false && filter_var($sanitizedUrl, FILTER_VALIDATE_URL) !== false) {
                        nitropack_sdk_purge_local($sanitizedUrl);
                    }
                } else if ($validatedUrl = esc_url($_POST["url"], array("http", "https"), "notdisplay")) {
                    nitropack_sdk_purge_local($validatedUrl);
                }
            } else {
                nitropack_sdk_purge_local();
            }
            break;
        }
    }
    exit;
}

function nitropack_passes_page_requirements() {
    return !(
        is_preview() ||
        is_robots() ||
        is_feed() ||
        is_comment_feed() ||
        is_trackback() ||
        is_user_logged_in() ||
        is_search() ||
        nitropack_is_ajax() ||
        nitropack_is_post() ||
        ( defined('WC_PLUGIN_FILE') && (is_page( 'cart' ) || is_cart()) ) || // WooCommerce
        ( defined('WC_PLUGIN_FILE') && (is_page( 'checkout' ) || is_checkout()) ) || // WooCommerce
        ( defined('WC_PLUGIN_FILE') && is_account_page() ) // WooCommerce
    );
}

function nitropack_is_ajax() {
    return (function_exists("wp_doing_ajax") && wp_doing_ajax()) || (defined('DOING_AJAX') && DOING_AJAX) || (!empty($_SERVER["HTTP_X_REQUESTED_WITH"]) && $_SERVER["HTTP_X_REQUESTED_WITH"] == "XMLHttpRequest");
}

function nitropack_is_post() {
    return (!empty($_SERVER['REQUEST_METHOD']) && $_SERVER['REQUEST_METHOD'] === 'POST') || !empty($_POST);
}

function nitropack_init() {
    header('Cache-Control: no-cache');
    header('X-Nitro-Cache: MISS');
    $GLOBALS["NitroPack.tags"] = array();

    if (is_valid_nitropack_webhook()) {
        nitropack_handle_webhook();
    } else {
        if (!isset($_GET["wpf_action"]) && nitropack_passes_cookie_requirements() && nitropack_passes_page_requirements()) {
            $is_optimizer_request = isset($_SERVER["HTTP_X_NITROPACK_REQUEST"]);
            $is_warmup_request = !empty($_SERVER["HTTP_X_NITRO_WARMUP"]);
            $is_lighthouse_request = !empty($_SERVER["HTTP_USER_AGENT"]) && stripos($_SERVER["HTTP_USER_AGENT"], "lighthouse") !== false;
            $is_gtmetrix_request = !empty($_SERVER["HTTP_USER_AGENT"]) && stripos($_SERVER["HTTP_USER_AGENT"], "gtmetrix") !== false;

            if ($is_warmup_request) {
                nitropack_check_remote_cache(false); // Only ping the API letting our service know that this page must be cached.
                exit; // No need to continue handling this request. The response is not important.
            } else  if ($is_lighthouse_request || $is_gtmetrix_request) {
                nitropack_check_remote_cache(); // Only ping the API letting our service know that this page must be cached.
            } else {
                add_action('wp_footer', 'nitropack_print_beacon_script');
            }

            $active_plugins = apply_filters('active_plugins', get_option('active_plugins'));
            if (in_array('woocommerce-multilingual/wpml-woocommerce.php', $active_plugins, true) && (!isset($_COOKIE["np_wc_currency"]) || !isset($_COOKIE["np_wc_currency_language"]))) {
                $final_action_before_output = 'wp_footer';
                add_action($final_action_before_output, 'set_wc_cookies');
            }

            if ($is_optimizer_request) { // Only care about tags for requests coming from our service. There is no need to do an API request when handling a standard client request.
                $nextPostTime = NULL;
                $scheduledPostsQuery = new WP_Query(array( 
                    'post_status' => 'future',
                    'posts_per_page' => 1,
                    'orderby' => 'date',
                    'order' => 'ASC'
                ));
                if ($scheduledPostsQuery->have_posts()) {
                    $nextPostTime = strtotime($scheduledPostsQuery->posts[0]->post_date);
                }

                // The Events Calendar compatibility
                if (defined('TRIBE_EVENTS_FILE') && function_exists('tribe_get_events')) {
                    $events = tribe_get_events(array(
                        "posts_per_page" => 1,
                        "start_date" => time()
                    ));

                    if (count($events)) {
                        $nextEventTime = strtotime($events[0]->event_date);
                        if (!$nextPostTime || $nextEventTime < $nextPostTime) {
                            $nextPostTime = $nextEventTime;
                        }
                    }
                }

                if ($nextPostTime) {
                    header("X-Nitro-Expires: " . $nextPostTime);
                }

                add_action("parse_query", "nitropack_handle_page_tags");
                add_action('the_post', 'nitropack_handle_the_post');
                add_action('wp_footer', 'nitropack_log_tags');
            }
        } else {
            header("X-Nitro-Disabled: 1");
        }
    }
}

function nitropack_print_beacon_script() {
    echo nitropack_get_beacon_script();
}

function nitropack_get_beacon_script() {
    $siteConfig = nitropack_get_site_config();
    if ($siteConfig && !empty($siteConfig["siteId"]) && !empty($siteConfig["siteSecret"])) {
        if (null !== $nitro = get_nitropack_sdk($siteConfig["siteId"], $siteConfig["siteSecret"]) ) {
            $url = $nitro->getUrl();
            if (function_exists("hash_hmac") && function_exists("hash_equals")) {
                $hash = hash_hmac("sha512", $url, $siteConfig["siteSecret"]);
            } else {
                $hash = "";
            }
            $url = base64_encode($url); // We want only ASCII

            echo "<script nitro-exclude>if (document.cookie.indexOf('nitroCache=HIT') == -1) {var nitroData = new FormData(); nitroData.append('nitroBeaconUrl', '$url'); nitroData.append('nitroBeaconHash', '$hash'); navigator.sendBeacon(location.href, nitroData)};</script>";
        }
    }
}

function nitropack_has_advanced_cache() {
    return defined( 'NITROPACK_ADVANCED_CACHE' );
}

function set_wc_cookies() {
    $wcCurrency = WC()->session->get("client_currency");
    $wcCurrencyLanguage = WC()->session->get("client_currency_language");
    if (!$wcCurrency) $wcCurrency = 0;
    if (!$wcCurrencyLanguage) $wcCurrencyLanguage = 0;
    setcookie('np_wc_currency', $wcCurrency, time() + (86400 * 7), "/");
    setcookie('np_wc_currency_language', $wcCurrencyLanguage, time() + (86400 * 7), "/");
}

function nitropack_check_remote_cache($serve_cache = false) {
    static $isCacheRequested = false;

    if ($isCacheRequested) return;
    if (is_preview() || is_404()) return;

    foreach (headers_list() as $header) {
        if (stripos(trim($header), "location") === 0) {
            return; // This will be a redirect, no need to check for cache
        }
    }

    if (!empty($GLOBALS["NitroPack.instance"])) {
        $nitro = $GLOBALS["NitroPack.instance"];
        $isCacheRequested = true;

        $layout = nitropack_get_layout();
        if (defined("ICL_LANGUAGE_CODE")) {
            $layout .= "-" . ICL_LANGUAGE_CODE; // Separate critical CSS based on the current WPML language
        }

        try {
            //do_action('nitropack_integration_purge_url', $nitro->getUrl()); // TODO: Do this when handling the cache_ready webhook. Currently this will only work for server setups where the response can be flushed to the client before reaching this point.
        } catch (\Exception $e) {
            // Exception while signaling our 3rd party integrations to purge their cache
        }

        try {
            if ($nitro->hasRemoteCache($layout) && $serve_cache === true) {
                header('X-Nitro-Cache: HIT');
                $nitro->pageCache->readfile();
                exit;
            } else {
                $GLOBALS["NitroPack.tags"]["pageType:" . $layout] = 1;
            }
        } catch (\Exception $e) {}
    }
}

function nitropack_handle_page_tags() {
    $layout = nitropack_get_layout();
    
    if ($layout == "category") {
        $catId = get_query_var('cat');
        $GLOBALS["NitroPack.tags"]["category:" . $catId] = 1;
    }
}

function nitropack_validate_site_id($siteId) {
    if (!preg_match("/^([a-zA-Z]{32})$/", trim($siteId), $matches)) {
        return false;
    }
    return $matches[1];
}

function nitropack_validate_site_secret($siteSecret) {
    if (!preg_match("/^([a-zA-Z0-9]{64})$/", trim($siteSecret), $matches)) {
        return false;
    }
    return $matches[1];
}

function nitropack_validate_webhook_token($token) {
    return preg_match("/^([abcdef0-9]{32})$/", strtolower($token));
}

function nitropack_validate_wc_currency($cookieValue) {
    return preg_match("/^([a-z]{3})$/", strtolower($cookieValue));
}

function nitropack_validate_wc_currency_language($cookieValue) {
    return preg_match("/^([a-z_\\-]{2,})$/", strtolower($cookieValue));
}

/** Step 3. */
function nitropack_options() {
    if ( !current_user_can( 'manage_options' ) )  {
        wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
    }

    wp_enqueue_style('nitropack_bootstrap_css', plugin_dir_url(__FILE__) . 'view/stylesheet/bootstrap.min.css');
    wp_enqueue_style('nitropack_css', plugin_dir_url(__FILE__) . 'view/stylesheet/nitropack.css');
    wp_enqueue_style('nitropack_font-awesome_css', plugin_dir_url(__FILE__) . 'view/stylesheet/fontawesome/font-awesome.min.css', true);
    wp_enqueue_script('nitropack_bootstrap_js', plugin_dir_url(__FILE__) . 'view/javascript/bootstrap.min.js', true);
    wp_enqueue_script('nitropack_notices_js', plugin_dir_url(__FILE__) . 'view/javascript/np_notices.js', true);
    wp_enqueue_script('nitropack_overlay_js', plugin_dir_url(__FILE__) . 'view/javascript/overlay.js', true);
    wp_enqueue_script('nitropack_embed_js', 'https://nitropack.io/asset/js/embed.js', true);
    wp_enqueue_script( 'jquery-form' );

    $siteId = esc_attr( get_option('nitropack-siteId') );
    $siteSecret = esc_attr( get_option('nitropack-siteSecret') );
    $enableCompression = get_option('nitropack-enableCompression');
    $checkedCompression = get_option('nitropack-checkedCompression');

    if (empty($siteId) || empty($siteSecret)) {
        include plugin_dir_path(__FILE__) . nitropack_trailingslashit('view') . 'connect.php';
    } else {
        $planDetailsUrl = get_nitropack_integration_url("plan_details_json");
        $optimizationDetailsUrl = get_nitropack_integration_url("optimization_details_json");
        $quickSetupUrl = get_nitropack_integration_url("quicksetup_json");
        $quickSetupSaveUrl = get_nitropack_integration_url("quicksetup");
        
        include plugin_dir_path(__FILE__) . nitropack_trailingslashit('view') . 'admin.php';
    }
}

function nitropack_is_connected() {
    $siteId = esc_attr( get_option('nitropack-siteId') );
    $siteSecret = esc_attr( get_option('nitropack-siteSecret') );
    return !empty($siteId) && !empty($siteSecret);
}

function nitropack_print_notice($type, $message) {
    echo '<div class="notice notice-' . $type . ' is-dismissible">';
    echo '<p><strong>NitroPack:</strong> ' . $message . '</p>';
    echo '</div>';
}

function nitropack_admin_notices() {
    if (!empty($_COOKIE["nitropack_after_activate_notice"])) {
        setcookie("nitropack_after_activate_notice", 1, time() - 15);
        nitropack_print_notice("info", "<script>document.cookie = 'nitropack_after_activate_notice=1; expires=Thu, 01 Jan 1970 00:00:01 GMT;';</script>NitroPack has been successfully activated, but it is not connected yet. Please go to <a href='" . admin_url( 'options-general.php?page=nitropack' ) . "'>its settings</a> page to connect it in order to start opitmizing your site!");
    }

    if (defined('BREEZE_PLUGIN_DIR')) { // Breeze cache plugin
        $clashingPlugin = "Breeze";
        nitropack_print_notice("warning", "It seems like $clashingPlugin is active. NitroPack and $clashingPlugin are both caching plugins and can interfere with each other. Please deactivate $clashingPlugin for best results in NitroPack.");
    }

    if (defined('WP_ROCKET_VERSION')) { // WP-Rocket
        $clashingPlugin = "WP-Rocket";
        nitropack_print_notice("warning", "It seems like $clashingPlugin is active. NitroPack and $clashingPlugin are both caching plugins and can interfere with each other. Please deactivate $clashingPlugin for best results in NitroPack.");
    }

    if (defined('W3TC')) { // W3 Total Cache
        $clashingPlugin = "W3 Total Cache";
        nitropack_print_notice("warning", "It seems like $clashingPlugin is active. NitroPack and $clashingPlugin are both caching plugins and can interfere with each other. Please deactivate $clashingPlugin for best results in NitroPack.");
    }

    if (defined('WPFC_MAIN_PATH')) { // WP Fastest Cache
        $clashingPlugin = "WP Fastest Cache";
        nitropack_print_notice("warning", "It seems like $clashingPlugin is active. NitroPack and $clashingPlugin are both caching plugins and can interfere with each other. Please deactivate $clashingPlugin for best results in NitroPack.");
    }

    if (defined('PHASTPRESS_VERSION')) { // PhastPress
        $clashingPlugin = "PhastPress";
        nitropack_print_notice("warning", "It seems like $clashingPlugin is active. NitroPack and $clashingPlugin are both caching plugins and can interfere with each other. Please deactivate $clashingPlugin for best results in NitroPack.");
    }

    if (class_exists('Swift_Performance') || class_exists('Swift_Performance_Lite')) { // Swift Performance
        $clashingPlugin = "Swift Performance";
        nitropack_print_notice("warning", "It seems like $clashingPlugin is active. NitroPack and $clashingPlugin are both caching plugins and can interfere with each other. Please deactivate $clashingPlugin for best results in NitroPack.");
    }

    if (class_exists('PagespeedNinja')) { // PageSpeed Ninja
        $clashingPlugin = "PageSpeed Ninja";
        nitropack_print_notice("warning", "It seems like $clashingPlugin is active. NitroPack and $clashingPlugin are both caching plugins and can interfere with each other. Please deactivate $clashingPlugin for best results in NitroPack.");
    }

    if (defined('AUTOPTIMIZE_PLUGIN_VERSION')) { // Autoptimize
        $clashingPlugin = "Autoptimize";
        nitropack_print_notice("warning", "It seems like $clashingPlugin is active. NitroPack and $clashingPlugin have overlapping functionality and can interfere with each other. Please deactivate $clashingPlugin for best results in NitroPack.");
    }

    if (nitropack_is_connected()) {
        if (!nitropack_has_advanced_cache()) {
            $advancedCacheFile = nitropack_trailingslashit(WP_CONTENT_DIR) . 'advanced-cache.php';
            if (!file_exists($advancedCacheFile) || strpos(file_get_contents($advancedCacheFile), "NITROPACK_ADVANCED_CACHE") === false) { // For some reason we get the notice right after connecting (even though the advanced-cache file is already in place). This check works around this issue :(
                if (nitropack_install_advanced_cache()) {
                    nitropack_print_notice("info", "The file /wp-content/advanced-cache.php was either missing or not the one generated by NitroPack. NitroPack re-installed its version of the file, so it can function properly. Possibly there is another active page caching plugin in your system. For correct operation, please deactivate any other page caching plugins.");
                } else {
                    nitropack_print_notice("error", "The file /wp-content/advanced-cache.php cannot be created. This can lead to slower cache delivery. Please make sure that the /wp-content/ directory is writable and refresh this page.");
                }
            }
        }

        if ( (!defined("WP_CACHE") || !WP_CACHE) ) {
            if (nitropack_is_flywheel()) { // Flywheel: This is configured throught the FW control panel
                nitropack_print_notice("warning", "The WP_CACHE setting is not enabled. Please go to your FlyWheel control panel and enable this setting. You can find more information <a href='https://getflywheel.com/wordpress-support/how-to-enable-wp_cache/' target='_blank'>in this document</a>.");
            } else if (!nitropack_set_wp_cache_const(true)) {
                nitropack_print_notice("error", "The WP_CACHE constant cannot be set in the wp-config.php file. This can lead to slower cache delivery. Please make sure that the /wp-config.php file is writable and refresh this page.");
            }
        }

        if ( !nitropack_data_dir_exists() && !nitropack_init_data_dir()) {
            nitropack_print_notice("error", "The NitroPack data directory cannot be created. Please make sure that the /wp-content/ directory is writable and refresh this page.");
            return;
        }

        $siteId = esc_attr( get_option('nitropack-siteId') );
        $siteSecret = esc_attr( get_option('nitropack-siteSecret') );
        $blogId = get_current_blog_id();

        if ( !nitropack_config_exists() && !nitropack_update_current_blog_config($siteId, $siteSecret, $blogId)) {
            nitropack_print_notice("error", "The NitroPack static config file cannot be created. Please make sure that the /wp-content/nitropack/ directory is writable and refresh this page.");
        }
    }
}

function nitropack_filter_non_original_cookies(&$cookies) {
    global $np_originalRequestCookies;
    $ogNames = is_array($np_originalRequestCookies) ? array_keys($np_originalRequestCookies) : array();
    foreach ($cookies as $name=>$val) {
        if (!in_array($name, $ogNames)) {
            unset($cookies[$name]);
        }
    }
}

function get_nitropack_sdk($siteId = null, $siteSecret = null, $urlOverride = NULL) {
    $siteConfig = nitropack_get_site_config();

    require_once 'nitropack-sdk/autoload.php';
    $siteId = $siteId ? $siteId : ($siteConfig ? $siteConfig['siteId'] : get_option('nitropack-siteId'));
    $siteSecret = $siteSecret ? $siteSecret : ($siteConfig ? $siteConfig['siteSecret'] : get_option('nitropack-siteSecret'));

    if ($siteId && $siteSecret) {
        try {
            NitroPack\SDK\NitroPack::addCookieFilter("nitropack_filter_non_original_cookies");
            $dataDir = nitropack_trailingslashit(NITROPACK_DATA_DIR) . $siteConfig["siteId"]; // dir without a trailing slash, because this is how the SDK expects it
            $nitro = new NitroPack\SDK\NitroPack($siteId, $siteSecret, $_SERVER["HTTP_USER_AGENT"], $urlOverride, $dataDir);
        } catch (\Exception $e) {
            return NULL;
        }
        
        return $nitro;
    }

    return NULL;
}

function get_nitropack_integration_url($integration, $nitro = null) {
    if ($nitro || (null !== $nitro = get_nitropack_sdk()) ) {
        return $nitro->integrationUrl($integration);
    }

    return "#";
}

function register_nitropack_settings() {
    register_setting( NITROPACK_OPTION_GROUP, 'nitropack-siteId', array('show_in_rest' => false) );
    register_setting( NITROPACK_OPTION_GROUP, 'nitropack-siteSecret', array('show_in_rest' => false) );
    register_setting( NITROPACK_OPTION_GROUP, 'nitropack-enableCompression', array('default' => -1) );
}

function nitropack_get_layout() {
    $layout = "default";

    if (is_home()) {
        $layout = "home";
    } else if (is_page()) {
        $layout = "page";
    } else if (is_attachment()) {
        $layout = "attachment";
    } else if (is_single()) {
        $layout = "post";
    } else if (is_category()) {
        $layout = "category";
    } else if (is_archive()) {
        $layout = "archive";
    } else if (is_author()) {
        $layout = "author";
    } else if (is_search()) {
        $layout = "search";
    }

    return $layout;
}

function nitropack_sdk_invalidate($url = NULL, $tag = NULL, $reason = NULL) {
    if (null !== $nitro = get_nitropack_sdk()) {
        try {
            $siteConfig = nitropack_get_site_config();
            $homeUrl = $siteConfig && !empty($siteConfig["home_url"]) ? $siteConfig["home_url"] : get_home_url();

            $nitro->invalidateCache(NULL, "pageType:home", $reason);
            $nitro->invalidateCache($url, $tag, $reason);

            try {
                do_action('nitropack_integration_purge_url', $homeUrl);

                if ($tag) {
                    do_action('nitropack_integration_purge_all');
                } else if ($url) {
                    do_action('nitropack_integration_purge_url', $url);
                } else {
                    do_action('nitropack_integration_purge_all');
                }
            } catch (\Exception $e) {
                // Exception while signaling our 3rd party integration addons to purge their cache
            }
        } catch (\Exception $e) {
            return false;
        }

        return true;
    }

    return false;
}

function nitropack_sdk_purge($url = NULL, $reason = NULL, $type = \NitroPack\SDK\PurgeType::COMPLETE) {
    if (null !== $nitro = get_nitropack_sdk()) {
        try {
            $nitro->invalidateCache(NULL, "pageType:home", NULL);
            $nitro->purgeCache($url, NULL, $type, $reason);

            try {
                $siteConfig = nitropack_get_site_config();
                $homeUrl = $siteConfig && !empty($siteConfig["home_url"]) ? $siteConfig["home_url"] : get_home_url();
                do_action('nitropack_integration_purge_url', $homeUrl);

                if ($url) {
                    do_action('nitropack_integration_purge_url', $url);
                } else {
                    do_action('nitropack_integration_purge_all');
                }
            } catch (\Exception $e) {
                // Exception while signaling our 3rd party integration addons to purge their cache
            }
        } catch (\Exception $e) {
            return false;
        }

        return true;
    }

    return false;
}

function nitropack_sdk_purge_local($url = NULL) {
    if (null !== $nitro = get_nitropack_sdk()) {
        try {
            if ($url) {
                $nitro->purgeLocalUrlCache($url);
                do_action('nitropack_integration_purge_url', $url);
            } else {
                $nitro->purgeLocalCache();

                try {
                    do_action('nitropack_integration_purge_all');
                } catch (\Exception $e) {
                    // Exception while signaling our 3rd party integration addons to purge their cache
                }
            }
        } catch (\Exception $e) {
            return false;
        }

        return true;
    }

    return false;
}

function nitropack_fetch_config() {
    if (null !== $nitro = get_nitropack_sdk()) {
        try {
            $nitro->fetchConfig();
        } catch (\Exception $e) {}
    }
}

function nitropack_switch_theme() {
    try {
        nitropack_sdk_purge(NULL, 'Theme switched'); // purge entire cache
    } catch (\Exception $e) {}
}

function nitropack_purge_cache() {
    try {
        if (nitropack_sdk_purge(NULL, 'Manual purge of all pages')) {
            nitropack_json_and_exit(array(
                "type" => "success",
                "message" => "Success! Cache has been purged successfully!"
            ));
        }
    } catch (\Exception $e) {}

    nitropack_json_and_exit(array(
        "type" => "error",
        "message" => "Error! There was an error and the cache was not purged!"
    ));
}

function nitropack_invalidate_cache() {
    try {
        if (nitropack_sdk_invalidate(NULL, NULL, 'Manual invalidation of all pages')) {
            nitropack_json_and_exit(array(
                "type" => "success",
                "message" => "Success! Cache has been invalidated successfully!"
            ));
        }
    } catch (\Exception $e) {}

    nitropack_json_and_exit(array(
        "type" => "error",
        "message" => "Error! There was an error and the cache was not invalidated!"
    ));
}

function nitropack_json_and_exit($array) {
    echo json_encode($array);
    exit;
}

function nitropack_clean_post_cache($post) {
    try {
        $postID = $post->ID;
        $postType = isset($post->post_type) ? $post->post_type : "post";

        nitropack_sdk_invalidate(NULL, "post:$postID", sprintf("Updated %s #%s", $postType, $postID));
        $posts = get_post_ancestors($postID);
        foreach ($posts as $parentID) {
            nitropack_sdk_invalidate(NULL, "post:$parentID", sprintf("Updated %s #%s", $postType, $postID));
        }
    } catch (\Exception $e) {}
}

function nitropack_handle_post_transition($new, $old, $post) {
    try {
        if ($old === "new" && $new === "auto-draft") { // Creating a new post, don't do anything for now. 
            return;
        }

        $invalidateAllPostTypes = array(
            "post" => "Post",
            "page" => "Page",
            "tribe_events" => "Calendar Event",
        );
        $postType = isset($post->post_type) ? $post->post_type : "post";
        $shouldInvalidateAll = in_array($postType, array_keys($invalidateAllPostTypes));

        if ($new == "future" && $shouldInvalidateAll) {
            nitropack_sdk_invalidate(NULL, NULL, sprintf("Invalidation of all pages due to scheduling %s #%s", $invalidateAllPostTypes[$postType], $post->ID));
        } else if ($new == "publish" && $old != "publish" && $shouldInvalidateAll) {
            nitropack_sdk_invalidate(NULL, NULL, sprintf("Invalidation of all pages due to publishing new %s #%s", $invalidateAllPostTypes[$postType], $post->ID));
        } else {
            nitropack_clean_post_cache($post);
        }
    } catch (\Exception $e) {
        // TODO: Log the error
    }
}

function nitropack_handle_the_post($post) {
    $GLOBALS["NitroPack.tags"]["post:" . $post->ID] = 1;
    $GLOBALS["NitroPack.tags"]["author:" . $post->post_author] = 1;
}

function nitropack_log_tags() {
    if (!empty($GLOBALS["NitroPack.instance"]) && !empty($GLOBALS["NitroPack.tags"])) {
        $nitro = $GLOBALS["NitroPack.instance"];
        $layout = nitropack_get_layout();
        try {
            if ($layout == "home") {
                $nitro->getApi()->tagUrl($nitro->getUrl(), "pageType:home");
            } else {
                $nitro->getApi()->tagUrl($nitro->getUrl(), array_keys($GLOBALS["NitroPack.tags"]));
            }
        } catch (\Exception $e) {}
    }
}

function nitropack_verify_connect() {
    if (empty($_POST["siteId"]) || empty($_POST["siteSecret"]) ||
        !($siteId = nitropack_validate_site_id(esc_attr($_POST["siteId"]))) ||
        !($siteSecret = nitropack_validate_site_secret(esc_attr($_POST["siteSecret"])))) {
        nitropack_json_and_exit(array("status" => "error", "message" => "Site ID and Site Secret cannot be empty"));
    }
    try {
        $blogId = get_current_blog_id();
        if (null !== $nitro = get_nitropack_sdk($siteId, $siteSecret)) {
            if ($nitro->fetchConfig()) {
                $token = md5(uniqid());
                update_option("nitropack-webhookToken", $token);
                update_option("nitropack-enableCompression", -1);

                $configUrl = new \NitroPack\Url(get_home_url() . "?nitroWebhook=config&token=$token");
                $cacheClearUrl = new \NitroPack\Url(get_home_url() . "?nitroWebhook=cache_clear&token=$token");
                $cacheReadyUrl = new \NitroPack\Url(get_home_url() . "?nitroWebhook=cache_ready&token=$token");

                $nitro->getApi()->setWebhook("config", $configUrl);
                $nitro->getApi()->setWebhook("cache_clear", $cacheClearUrl);
                $nitro->getApi()->setWebhook("cache_ready", $cacheReadyUrl);

                // _icl_current_language is WPML cookie, it is added here for compatibility with this module
                $customVariationCookies = array("np_wc_currency", "np_wc_currency_language", "_icl_current_language");
                $variationCookies = $nitro->getApi()->getVariationCookies();
                foreach ($variationCookies as $cookie) {
                    $index = array_search($cookie["name"], $customVariationCookies);
                    if ($index !== false) {
                        array_splice($customVariationCookies, $index, 1);
                    }
                }

                foreach ($customVariationCookies as $cookieName) {
                    $nitro->getApi()->setVariationCookie($cookieName);
                }

                $nitro->fetchConfig(); // Reload the variation cookies

                nitropack_update_current_blog_config($siteId, $siteSecret, $blogId);
                nitropack_install_advanced_cache();

                try {
                    do_action('nitropack_integration_purge_all');
                } catch (\Exception $e) {
                    // Exception while signaling our 3rd party integration addons to purge their cache
                }

                nitropack_event("connect", $nitro);
                nitropack_event("enable_extension", $nitro);

                nitropack_json_and_exit(array("status" => "success"));
            }
        }
    } catch (\Exception $e) {
        nitropack_json_and_exit(array("status" => "error", "message" => "Incorrect API credentials. Please make sure that you copied them correctly and try again."));
    }

    nitropack_json_and_exit(array("status" => "error"));
}

function nitropack_disconnect() {
    nitropack_uninstall_advanced_cache();
    nitropack_event("disconnect");
    nitropack_unset_current_blog_config();
    delete_option("nitropack-siteId");
    delete_option("nitropack-siteSecret");
}

function nitropack_set_compression_ajax() {
    $compressionStatus = !empty($_POST["compressionStatus"]);
    update_option("nitropack-enableCompression", (int)$compressionStatus);
}

function nitropack_test_compression_ajax() {
    $hasCompression = true;
    try {
        if (nitropack_is_flywheel()) { // Flywheel: Compression is enabled by default
            update_option("nitropack-enableCompression", 0);
        } else {
            require_once plugin_dir_path(__FILE__) . nitropack_trailingslashit('nitropack-sdk') . 'autoload.php';
            $http = new NitroPack\HttpClient(get_site_url());
            $http->setHeader("X-NitroPack-Request", 1);
            $http->timeout = 25;
            $http->fetch();
            $headers = $http->getHeaders();
            if (!empty($headers["content-encoding"]) && strtolower($headers["content-encoding"]) == "gzip") { // compression is present, so there is no need to enable it in NitroPack. We only check for GZIP, because this is the only supported compression in the HttpClient
                update_option("nitropack-enableCompression", 0);
                $hasCompression = true;
            } else { // no compression, we must enable it from NitroPack
                update_option("nitropack-enableCompression", 1);
                $hasCompression = false;
            }
        }
        update_option("nitropack-checkedCompression", 1);
    } catch (\Exception $e) {
        nitropack_json_and_exit(array("status" => "error"));
    }

    nitropack_json_and_exit(array("status" => "success", "hasCompression" => $hasCompression));
}

function nitropack_handle_compression_toggle() {
    $new_value = get_option("nitropack-enableCompression"); // Unfortunately WordPress is not feeding in the $new_value as a function argument, even though the documentation says it should be. This is why we have to do this ugly thing...
    nitropack_update_blog_compression($new_value == 1);
}

function nitropack_update_blog_compression($enableCompression = false) {
    if (nitropack_is_connected()) {
        $siteId = esc_attr( get_option('nitropack-siteId') );
        $siteSecret = esc_attr( get_option('nitropack-siteSecret') );
        $blogId = get_current_blog_id();
        nitropack_update_current_blog_config($siteId, $siteSecret, $blogId, $enableCompression);
    }
}

function nitropack_enable_warmup() {
    if (null !== $nitro = get_nitropack_sdk()) {
        try {
            $nitro->getApi()->enableWarmup();
            $nitro->getApi()->setWarmupHomepage(get_home_url());
            $nitro->getApi()->runWarmup();
        } catch (\Exception $e) {
        }

        nitropack_json_and_exit(array(
            "type" => "success",
            "message" => "Success! Cache warmup has been enabled successfully!"
        ));
    }

    nitropack_json_and_exit(array(
        "type" => "error",
        "message" => "Error! There was an error while enabling the cache warmup!"
    ));
}

function nitropack_disable_warmup() {
    if (null !== $nitro = get_nitropack_sdk()) {
        try {
            $nitro->getApi()->disableWarmup();
            $nitro->getApi()->resetWarmup();
        } catch (\Exception $e) {
        }

        nitropack_json_and_exit(array(
            "type" => "success",
            "message" => "Success! Cache warmup has been disabled successfully!"
        ));
    }

    nitropack_json_and_exit(array(
        "type" => "error",
        "message" => "Error! There was an error while disabling the cache warmup!"
    ));
}

function nitropack_run_warmup() {
    if (null !== $nitro = get_nitropack_sdk()) {
        try {
            $nitro->getApi()->runWarmup();
        } catch (\Exception $e) {
        }

        nitropack_json_and_exit(array(
            "type" => "success",
            "message" => "Success! Cache warmup has been started successfully!"
        ));
    }

    nitropack_json_and_exit(array(
        "type" => "error",
        "message" => "Error! There was an error while starting the cache warmup!"
    ));
}

function nitropack_estimate_warmup() {
    if (null !== $nitro = get_nitropack_sdk()) {
        try {
            if (!session_id()) {
                session_start();
            }
            $id = !empty($_POST["estId"]) ? $_POST["estId"] : NULL;
            if ($id !== NULL && $id != $_SESSION["nitroEstimateId"]) {
                nitropack_json_and_exit(array(
                    "type" => "error",
                    "message" => "Error! Invalid estimation ID!"
                ));
            }

            $nitro->getApi()->setWarmupHomepage(get_home_url());
            $optimizationsEstimate = $nitro->getApi()->estimateWarmup($id);

            if ($id === NULL) {
                $_SESSION["nitroEstimateId"] = $optimizationsEstimate; // When id is NULL, $optimizationsEstimate holds the ID for the newly started estimate
            }
        } catch (\Exception $e) {
        }

        nitropack_json_and_exit(array(
            "type" => "success",
            "res" => $optimizationsEstimate
        ));
    }

    nitropack_json_and_exit(array(
        "type" => "error",
        "message" => "Error! There was an error while estimating the cache warmup!"
    ));
}

function nitropack_warmup_stats() {
    if (null !== $nitro = get_nitropack_sdk()) {
        try {
            $stats = $nitro->getApi()->getWarmupStats();
        } catch (\Exception $e) {
        }

        nitropack_json_and_exit(array(
            "type" => "success",
            "stats" => $stats
        ));
    }

    nitropack_json_and_exit(array(
        "type" => "error",
        "message" => "Error! There was an error while fetching warmup stats!"
    ));
}

function nitropack_data_dir_exists() {
    return defined("NITROPACK_DATA_DIR") && is_dir(NITROPACK_DATA_DIR);
}

function nitropack_init_data_dir() {
    return nitropack_data_dir_exists() || @mkdir(NITROPACK_DATA_DIR, 0755, true);
}

function nitropack_config_exists() {
    return defined("NITROPACK_CONFIG_FILE") && file_exists(NITROPACK_CONFIG_FILE);
}

function nitropack_get_site_config() {
    $siteConfig = null;
    $npConfig = nitropack_get_config();
    $currentUrl = $_SERVER["HTTP_HOST"] . $_SERVER["REQUEST_URI"];
    $matchLength = 0;
    foreach ($npConfig as $siteUrl => $config) {
        if (strpos($currentUrl, $siteUrl) === 0 && strlen($siteUrl) > $matchLength) {
            $siteConfig = $config;
            $matchLength = strlen($siteUrl);
        }
    }

    return $siteConfig;
}

function nitropack_set_config($config) {
    if (!nitropack_data_dir_exists() && !nitropack_init_data_dir()) return false;
    $GLOBALS["nitropack.config"] = $config;
    return WP_DEBUG ? file_put_contents(NITROPACK_CONFIG_FILE, json_encode($config, JSON_PRETTY_PRINT)) : @file_put_contents(NITROPACK_CONFIG_FILE, json_encode($config, JSON_PRETTY_PRINT));
}

function nitropack_get_config() {
    if (!empty($GLOBALS["nitropack.config"])) {
        return $GLOBALS["nitropack.config"];
    }

    $config = array();

    if (nitropack_config_exists()) {
        $config = json_decode(file_get_contents(NITROPACK_CONFIG_FILE), true);
    }

    $GLOBALS["nitropack.config"] = $config;
    return $config;
}

function nitropack_update_current_blog_config($siteId, $siteSecret, $blogId, $enableCompression = null) {
    if ($enableCompression === null) {
        $enableCompression = (get_option('nitropack-enableCompression') == 1);
    }

    $webhookToken = get_option('nitropack-webhookToken');
    $hosting = nitropack_detect_hosting();

    $home_url = get_home_url();
    $configKey = preg_replace("/^https?:\/\/(.*)/", "$1", $home_url);
    $staticConfig = nitropack_get_config();
    $staticConfig[$configKey] = array(
        "siteId" => $siteId,
        "siteSecret" => $siteSecret,
        "blogId" => $blogId,
        "compression" => $enableCompression,
        "webhookToken" => $webhookToken,
        "home_url" => $home_url,
        'hosting' => $hosting
    );
    return nitropack_set_config($staticConfig);
}

function nitropack_unset_current_blog_config() {
    $home_url = get_home_url();
    $configKey = preg_replace("/^https?:\/\/(.*)/", "$1", $home_url);
    $staticConfig = nitropack_get_config();
    if (!empty($staticConfig[$configKey])) {
        unset($staticConfig[$configKey]);
        return nitropack_set_config($staticConfig);
    }

    return true;
}

function nitropack_event($event, $nitro = null) {
    global $wp_version;

    try {
        $eventUrl = get_nitropack_integration_url("extensionEvent", $nitro);
        $domain = !empty($_SERVER["HTTP_HOST"]) ? $_SERVER["HTTP_HOST"] : "Unknown";

        $query_data = array(
            'event' => $event,
            'platform' => 'WordPress',
            'platform_version' => $wp_version,
            'nitropack_extension_version' => NITROPACK_VERSION,
            'additional_meta_data' => "{}",
            'domain' => $domain
        );

        $client = new NitroPack\HttpClient($eventUrl . '&' . http_build_query($query_data));
        $client->doNotDownload = true;
        $client->fetch();
    } catch (\Exception $e) {}
}

function nitropack_is_flywheel() {
    return defined("FLYWHEEL_PLUGIN_DIR");
}

function nitropack_is_cloudways() {
    return array_key_exists("cw_allowed_ip", $_SERVER) || preg_match("~/home/.*?cloudways.*~", __FILE__);
}

function nitropack_is_wpe() {
    return !!getenv('IS_WPE');
}

function nitropack_is_siteground() {
    $configFilePath = nitropack_trailingslashit(ABSPATH) . "wp-config.php";
    if (!file_exists($configFilePath)) {
        $configFilePath = nitropack_trailingslashit(dirname(ABSPATH)) . "wp-config.php";
        $settingsFilePath = nitropack_trailingslashit(dirname(ABSPATH)) . "wp-settings.php"; // We need to check for this file to avoid confusion if the current installation is a nested directory of another WP installation. Refer to wp-load.php for more information.
        if (!file_exists($configFilePath) || file_exists($settingsFilePath)) {
            return false;
        }
    }
    return strpos(file_get_contents($configFilePath), 'Added by SiteGround WordPress management system') !== false;
}

function nitropack_detect_hosting() {
    if (nitropack_is_flywheel()) {
        return "flywheel";
    } else if (nitropack_is_cloudways()) {
        return "cloudways";
    } else if (nitropack_is_wpe()) {
        return "wpengine";
    } else if (nitropack_is_siteground()) {
        return "siteground";
    } else {
        return "unknown";
    }
}

// Init integration action handlers
require_once 'integrations.php';
if (did_action('muplugins_loaded')) {
    nitropack_check_and_init_integrations();
} else {
    add_action('muplugins_loaded', 'nitropack_check_and_init_integrations');
}
