<?php

function nitropack_check_and_init_integrations() {
    $siteConfig = nitropack_get_site_config();
    if ($siteConfig && !empty($siteConfig["hosting"])) {
        $hosting = $siteConfig["hosting"];
    } else {
        $hosting = nitropack_detect_hosting();
    }

    switch ($hosting) {
    case "cloudways":
        add_action('nitropack_integration_purge_url', 'nitropack_cloudways_purge_url');
        add_action('nitropack_integration_purge_all', 'nitropack_cloudways_purge_all');
        break;
    case "flywheel":
        add_filter('nitropack_varnish_purger', 'nitropack_flywheel_varnish_instance');
        add_action('nitropack_integration_purge_url', 'nitropack_varnish_purge_url');
        add_action('nitropack_integration_purge_all', 'nitropack_varnish_purge_all');
        break;
    case "wpengine":
        add_action('nitropack_integration_purge_url', 'nitropack_wpe_purge_url');
        add_action('nitropack_integration_purge_all', 'nitropack_wpe_purge_all');
        break;
    case "siteground":
        add_action('nitropack_integration_purge_url', 'nitropack_siteground_purge_url');
        add_action('nitropack_integration_purge_all', 'nitropack_siteground_purge_all');
        break;
    default:
        break;
    }

    add_action('plugins_loaded', 'nitropack_init_late_integrations');
}

function nitropack_init_late_integrations() {
    if (defined('SHORTPIXEL_AI_VERSION')) { // ShortPixel
        if (nitropack_is_ajax()) {
            add_action('wp_print_scripts', 'nitropack_shortpixel_deactivate');
        }
    }
}

/** WP Engine **/
function nitropack_wpe_purge_url($url) {
    try {
        $handler = function($paths) use($url) {
            $wpe_path = parse_url($url, PHP_URL_PATH);
            $wpe_query = parse_url($url, PHP_URL_QUERY);
            $varnish_path = $wpe_path;
            if (!empty($wpe_query)) {
                $varnish_path .= '?' . $wpe_query;
            }
            if ($url && count($paths) == 1 && $paths[0] == ".*") {
                return array($varnish_path);
            }
            return $paths;
        };
        add_filter( 'wpe_purge_varnish_cache_paths', $handler );
        WpeCommon::purge_varnish_cache();
        remove_filter( 'wpe_purge_varnish_cache_paths', $handler );
    } catch (\Exception $e) {
        // WPE exception
    }
}

function nitropack_wpe_purge_all() {
    try {
        WpeCommon::purge_varnish_cache();
    } catch (\Exception $e) {
        // WPE exception
    }
}

/** Cloudways' Breeze plugin **/
function nitropack_cloudways_purge_url($url) {
    try {
        $purger = new \NitroPack\SDK\Integrations\Varnish(array("127.0.0.1"), "URLPURGE");
        $purger->purge($url);
    } catch (\Exception $e) {
        // Breeze exception
    }
}

function nitropack_cloudways_purge_all() {
    try {
        $homepage = home_url().'/.*';
        $purger = new \NitroPack\SDK\Integrations\Varnish(array("127.0.0.1"), "PURGE");
        $purger->purge($homepage);
    } catch (\Exception $e) {
        // Exception
    }
}

/** SiteGround - Even though they use Nginx we can communicate with it as if it was Varnish **/
function nitropack_siteground_purge_url($url) {
    $hosts = ['127.0.0.1'];
    $url = preg_replace("/^https:\/\//", "http://", $url);
    $purger = new \NitroPack\SDK\Integrations\Varnish($hosts, 'PURGE');
    $purger->purge($url);
    return true;
}

function nitropack_siteground_purge_all() {
    $siteConfig = nitropack_get_site_config();
    if ($siteConfig && !empty($siteConfig["home_url"])) {
        return nitropack_siteground_purge_url($siteConfig["home_url"]);
    }
    return false;
}

/** Flywheel Varnish **/
function nitropack_flywheel_varnish_instance($type) {
    return new \NitroPack\SDK\Integrations\Varnish(array('127.0.0.1'), 'PURGE');
}

/** Generic Varnish **/
function nitropack_varnish_generic_instance($type) {
    $varnishConfig = nitropack_get_varnish_settings();
    $purgeMethod = ($type == 'single') ? $varnishConfig->PurgeSingleMethod : $varnishConfig->PurgeAllMethod;
    if (empty($purgeMethod)) $purgeMethod = 'PURGE';
    return new \NitroPack\SDK\Integrations\Varnish($varnishConfig->Servers, $purgeMethod);
}

function nitropack_varnish_purge_url($url) {
    try {
        $purger = apply_filters('nitropack_varnish_purger', 'single');
        $purger->purge($url);
    } catch (\Exception $e) {
        // Exception encountered while trying to purge varnish cache
    }
}

function nitropack_varnish_purge_all() {
    try {
        $purger = apply_filters('nitropack_varnish_purger', 'all');
        if (function_exists("get_home_url")) {
            $home = get_home_url();
        } else {
            $siteConfig = nitropack_get_site_config();
            $home = "/";
            if ($siteConfig && !empty($siteConfig["home_url"])) {
                $home = $siteConfig["home_url"];
            }
        }
        $purger->purge($home);
    } catch (\Exception $e) {
        // Exception encountered while trying to purge varnish cache
    }
}

function nitropack_get_varnish_settings() {
    if (null !== $nitro = get_nitropack_sdk()) {
        $config = $nitro->getConfig();
        return !empty($config->CacheIntegrations) && $empty($config->CacheIntegrations->Varnish) ? $config->CacheIntegrations->Varnish : null;
    }

    return null;
}

/** ShortPixel **/
function nitropack_shortpixel_deactivate() {
    if (wp_script_is('spai-scripts')) {
        wp_dequeue_script('spai-scripts');
    }
}
